import { GraphAbstraction } from '../../core/dtos/graph.abstraction.dto';
import { ActionService } from '../../core/services/action.service';
export declare const makeActionFunctionalService: (actionService: ActionService) => Readonly<{
    findAll: (entitiesPatternDto: GraphAbstraction, page?: number | undefined, size?: number | undefined) => import("./DBAction").DBAction;
    findOne: (entitiesPatternDto: GraphAbstraction) => import("./DBAction").DBAction;
    create: (entitiesPatternDto: GraphAbstraction, targetAlias: string) => import("./DBAction").DBAction;
    update: (entitiesPatternDto: GraphAbstraction, targetAlias: string) => import("./DBAction").DBAction;
    delete: (entitiesPatternDto: GraphAbstraction, targetAlias: string) => import("./DBAction").DBAction;
    runCypher: (cypher: string, parameters?: any) => import("./DBAction").CypherAction;
}>;
/**
 * Create a find all action that can be executed or used to generate a cypher string
 * The action will retrieve all instances matching the pattern given on the entities pattern dto
 * @param {GraphAbstraction} entitiesPatternDto - a dto which abstracts nodes and relationships
 * @param {number} [page] - If you want to use paginatiion you can send the page number to retrieve data
 * @param {number} [size] - If you want to use pagination you can send the size of your page
 * @return {DBAction}
 */
export declare const findAll: (entitiesPatternDto: GraphAbstraction, page?: number | undefined, size?: number | undefined) => import("./DBAction").DBAction;
/**
 * create a find one action that can be executed or used to generate a cypher string
 * The action will retrieve the first record matching the pattern given on the entities pattern dto
 * @param {GraphAbstraction} entitiesPatternDto - a dto which abstracts nodes and relationships
 * @return {DBAction}
 */
export declare const findOne: (entitiesPatternDto: GraphAbstraction) => import("./DBAction").DBAction;
/**
 * Make a create action that can be executed or used to generate a cypher string
 * The action will create an entitie in the DB using the alias of one as the target from the entities defined on the graph abstraction
 * @param {GraphAbstraction} entitiesPatternDto - a dto which abstracts nodes and relationships
 * @param {string} targetAlias - reference to an alias used on the entitiesPatternDto, this will be the entitie that will be created, if the entitie is a node and has relationships, the relationships will be generated too.
 * @return {DBAction}
 */
export declare const create: (entitiesPatternDto: GraphAbstraction, targetAlias: string) => import("./DBAction").DBAction;
/**
 * Make an update action that can be executed or used to generate a cypher string
 * The action will create an entitie in the DB using the alias of one as the target from the entities defined on the graph abstraction
 * @param {GraphAbstraction} entitiesPatternDto - a dto wich abstracts nodes and relationships
 * @param {string} targetAlias - reference to an alias used on the entitiesPatternDto, this will be the entitie that will be updated, all properties will be used to set properties if you want to set filters you have to defined using isFilter as true
 * @return {DBAction}
 */
export declare const update: (entitiesPatternDto: GraphAbstraction, targetAlias: string) => import("./DBAction").DBAction;
/**
 * Make a Delete action that can be executed or used to generate a cypher string
 * The action will delete the selected entitie in the DB using the alias of one as the target from the entities defined on the graph abstraction
 * @param {GraphAbstraction} entitiesPatternDto - a dto wich abstracts nodes and relationships
 * @param {string} targetAlias - reference to an alias used on the entitiesPatternDto, this will be the entitie that will be deleted
 * @return {DBAction}
 */
export declare const deleteAction: (entitiesPatternDto: GraphAbstraction, targetAlias: string) => import("./DBAction").DBAction;
/**
 * Run a raw cypher with parameters
 * @param {string} cypher - the cypher to be executed
 * @param {string} [parameter] - parameters to be used on the cypher execution
 */
export declare const runCypher: (cypher: string, parameters?: any) => import("./DBAction").CypherAction;
